#include <p18F2550.h>
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <delays.h>



//-------------------------------------------------------------------------------------------------------------------------------------------------------------
// PIC18F2550 Configuration Bit Settings
#pragma config FOSC = HS                           //XT_oscillator - 20Mhz - C.M.=4*(1/20)=0.2us
#pragma config	FCMEN = OFF			 //no fail safe clock monitor
#pragma config	IESO = OFF			  //oscillator switchover disabled
#pragma config PWRT = ON                          //oscillator power up timer enabled (release version only)
#pragma config BOR = OFF
#pragma config	WDT = OFF			 //watchdog timer disabled
#pragma config	MCLRE = ON  		        //MCLR pin enabled
#pragma config	LVP = OFF  			  //low voltage programming disabled
#pragma config	XINST = OFF                    //do not allow PIC18 extended instructions
#pragma config  STVREN = ON                   //stack overflow will cause reset
#pragma config	CP0 = OFF
#pragma config	CP1 = OFF
#pragma config	CP2 = OFF
#pragma config	CP3 = OFF
#pragma config	CPB = OFF
//-------------------------------------------------------------------------------------------------------------------------------------------------------------



//===================================================================================================================================
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ LCD FUNCTION ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//===================================================================================================================================

#define _XTAL_FREQ  (20000000UL)

typedef unsigned char uint8_t;
typedef unsigned short uint16_t;

void _delay(unsigned long cy)
{
    if (cy > 200000UL) Delay10KTCYx(cy / 10000);
    else if (cy > 20000) Delay1KTCYx(cy / 1000);
    else if (cy > 2000) Delay100TCYx(cy / 100);
    else Delay10TCYx(cy / 10);
}

#define __delay_us(x) _delay((unsigned long)((x)*(_XTAL_FREQ/4000000.0)))
#define __delay_ms(x) _delay((unsigned long)((x)*(_XTAL_FREQ/4000.0)))

#define MASKIN(reg,mask,x) do{reg^=(reg^(x))&(mask);}while(0)
//-------------------------------------------------------------------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------------------------------------------------------------------
#define LCD_TRIS              TRISB
#define LCD_LAT                LATB

#define LCD_BL_PIN          LATBbits.LATB7
#define LCD_RS_PIN          LATBbits.LATB4
#define LCD_E_PIN            LATBbits.LATB5

#define LCD_BL_TRIS        TRISBbits.TRISB7
#define LCD_RS_TRIS        TRISBbits.TRISB4
#define LCD_E_TRIS          TRISBbits.TRISB5
//-------------------------------------------------------------------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------------------------------------------------------------------
void pulseE( void )
{
    __delay_us( 1 ); LCD_E_PIN = 1;
    __delay_us( 1 ); LCD_E_PIN = 0;
}


void setNibble( uint8_t lsb )
{
    MASKIN( LCD_LAT, 0x0F, lsb );
}


void lcdWrite( uint8_t rs, uint8_t byte )
{
    LCD_RS_PIN = rs;
    setNibble( byte >> 4 );
    pulseE();
    setNibble( byte );
    pulseE();
    __delay_us( 60 );
}


void lcdClear( void )
{
    lcdWrite( 0, 0b00000001 );
    __delay_ms( 4 );
}
//-------------------------------------------------------------------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------------------------------------------------------------------
void lcdInit( void )
{
    LCD_TRIS = 0xF0;
    LCD_LAT  = 0xF0;

    LCD_BL_PIN = 1; // enable backlight/power
    LCD_RS_PIN = 0;
    LCD_E_PIN = 0;

    LCD_BL_TRIS = 0;
    LCD_RS_TRIS = 0;
    LCD_E_TRIS = 0;

    __delay_ms( 20 ); // power-up delay

    setNibble( 0b0011 ); // set interface to 8-bit
    pulseE(); __delay_ms( 5 );
    pulseE(); __delay_ms( 1 );
    pulseE(); __delay_ms( 1 );

    setNibble( 0b0010 ); // set interface to 4-bit
    pulseE(); __delay_ms( 1 );

    lcdWrite( 0, 0b00101000 ); // 4-bit, 2 lines, 5x8
    lcdWrite( 0, 0b00001100 ); // display on, cursor off
    lcdWrite( 0, 0b00000110 ); // entry mode l-to-r, no shift
    lcdClear();
}
//-------------------------------------------------------------------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------------------------------------------------------------------
void lcdGoto( int col, int row )
{
    uint8_t address = col;
    if ( row ) address += 0x40;
    lcdWrite( 0, 0x80 | address );
}


void lcdOut( const char *s )
{
    while ( *s )
    lcdWrite( 1, *s++ );
}
//-------------------------------------------------------------------------------------------------------------------------------------------------------------



//===================================================================================================================================
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ DS18B20 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//===================================================================================================================================

#define DQ                  LATAbits.LATA0
#define DQ_TRIS        TRISAbits.TRISA0         // "DQ_TRIS = 0;" This must included in the main program

//-------------------------------------------------------------------------------------------------------------------------------------------------------------
int pdbyte;

void ow_reset()
{
DQ_TRIS = 0;                        // Set DQ pin as output
DQ = 1;                                  // Start with the line high
pdbyte = 0x00;                        // Clear the PD byte
__delay_us( 1 );                      // Drive Low for 1us
DQ = 0;
__delay_us( 500 );                  // Drive Low for 500us
DQ = 1;
__delay_us( 700 );                  // Release line and wait 70us for PD Pulse

DQ_TRIS = 1;                        // Set DQ pin as input
__delay_us( 1 );                      // Wait 1us
if( DQ == 0) 
{
    pdbyte = 0x01;                    // Set PDBYTE to 1 if get a PD Pulse
}
__delay_us( 400 );                  // Wait 400us after PD Pulse
}
//-------------------------------------------------------------------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------------------------------------------------------------------
int  x, count, temp,  iobyte = 0;

void dsrxbyte()
{
    count = 0x08;                          //Set count = 8

    for( x=0; x<8 ;x++ )
    {
        DQ_TRIS = 0;                        // Set DQ pin as output
        __delay_us( 1 );                      // Wait 1us
        DQ = 0;                                 // Bring DQ low for 6us
        __delay_us( 6 );
        DQ = 1;                                 // Bring DQ low for 4us
        __delay_us( 4 );

        DQ_TRIS = 1;                        // Set DQ pin as input
        __delay_us( 1 );                      // Wait 1us

        if( DQ == 1 )
        {
            iobyte = iobyte | ( 2^x );          // Logig OR
        }

        else
        {
            temp = ~ ( 2^x );
            iobyte = iobyte & temp;          // Logig AND
        }

    }

}
//-------------------------------------------------------------------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------------------------------------------------------------------
dstxbyte(void)
{
    count = 8;

    for( x=0; x<8 ;x++ )
    {
        DQ_TRIS = 0;                        // Set DQ pin as output
        __delay_us( 1 );                      // Wait 1us
        DQ = 0;                                 // Set DQ pin low
        __delay_us( 3 );                     // Wait 3us

        temp = iobyte & ( 2^x );           // Logig AND

        if( temp == ( 2^x ) )
        {
            DQ = 1;                             // Set DQ pin high
        }

        __delay_us( 60 );                   // Wait 60us
        DQ = 1;                                 // Set DQ pin high
        __delay_us( 2 );                     // Wait 2us
    }

}
//-------------------------------------------------------------------------------------------------------------------------------------------------------------





//===================================================================================================================================
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ MAIN PROGRAM ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//===================================================================================================================================

void main( void )
{
    int result,tempc,tempf = 10;
    char lcd_char[20];

    float tempe = 35.75;
    int  ip = (int) tempe;
    int  fp = (int) ((tempe - ip) * 100. + 0.5 );

//DS18B20 variables
    int srchrom = 0xF0;
    int rdrom = 0x33;
    int mtchrom = 0x55;
    int skprom = 0xCC;
    int picmsb, piclsb;
    int temperature;


    ADCON0 = 0x00;
    ADCON1 = 0x0F;
    CMCON = 0x07;                //Disable Comporators

    __delay_ms( 100 );

    lcdInit( );

    __delay_ms( 100 );

    lcdGoto( 0, 0 );
    sprintf( lcd_char, "Iron Electronics");
    lcdOut( lcd_char );


    while ( 1 )
    {
        ow_reset( );
        iobyte = skprom;
        dstxbyte( );
        iobyte = 0x69;
        dstxbyte( );
        iobyte = 0x0E;
        dstxbyte( );
        dsrxbyte( );
        iobyte = picmsb;
        dsrxbyte( );
        iobyte = piclsb;
        ow_reset( );

        temperature = temperature | piclsb;
        temperature = temperature | (picmsb << 8);

        __delay_ms( 100 );
        sprintf( lcd_char, "%d", temperature );
        lcdGoto( 0, 1 );
        lcdOut( lcd_char );
    }
}
















//    strcpypgm2ram( s, "Iron Electronics" );

//   sprintf( lcd_char, "%d.%02d", ip, fp );
//   lcdGoto( 0, 1 );
//   lcdOut(lcd_char);
//   sprintf( lcd_char, "%d C", count );
//   lcdGoto( 8, 1 );
//   lcdOut(lcd_char);



//         mytemperature = ds_get_temp(); // bit 0 represents .5 degress
//         mytemperature = mytemperature >> 1; // bit 0 now represents 1 degree
//










//                for ( i = 0; i < 100; i++ )
//            __delay_ms( 10 );









/*


#define OW_LAT   LATAbits.LATA2
#define OW_PIN   PORTAbits.RA2
#define OW_TRIS TRISAbits.TRISA2

#ifndef __DS18S20_H
#define __DS18S20_H

#include <ow.h>

char ds_present(void)
{
    return ow_reset();
}

char ds_get_temp(void)
{
    char temp_low;
    char temp_high;
    int temperature=0;

    ow_reset();
    Delay100TCYx(30);
    ow_write_byte(0xCC);
    Delay100TCYx(1);
    ow_write_byte(0x44);
    Delay10KTCYx(255);
    Delay10KTCYx(255);
    Delay10KTCYx(255);
    ow_reset();
    Delay100TCYx(30);
    ow_write_byte(0xCC);
    Delay100TCYx(1);
    ow_write_byte(0xBE);
    Delay100TCYx(1);
    temp_low = ow_read_byte();
    Delay10TCYx(1);
    temp_high = ow_read_byte();
    ow_reset();
    temperature = temperature | temp_low;
    temperature = temperature | (temp_high << 8);
    return temperature;
}

#endif
 *
 */