/*  Name     	: Lcd_General.c
 *  Purpose  	: General Lcd Functions for 4x20 LCD for any microcontroller
 *  Author   	: Sergulen YENIDOGAN
 *  Date START  : 13.07.2013
 *  Date END  	: 13.07.2013
 *  Website  	: 
 *  E mail		: syenidogan@gmail.com
 *  Revision 	: None
 */



// ****** WRITES 1 BYTE TO LCD ************

void lcd_write(unsigned char c)
{   
	if(c & 0x80) LCD_D7=1; else LCD_D7=0;
	if(c & 0x40) LCD_D6=1; else LCD_D6=0;
	if(c & 0x20) LCD_D5=1; else LCD_D5=0;
	if(c & 0x10) LCD_D4=1; else LCD_D4=0;
	LCD_STROBE;
	if(c & 0x08) LCD_D7=1; else LCD_D7=0;
	if(c & 0x04) LCD_D6=1; else LCD_D6=0;
	if(c & 0x02) LCD_D5=1; else LCD_D5=0;
	if(c & 0x01) LCD_D4=1; else LCD_D4=0;
	LCD_STROBE;	
	DelayUs(60);
}


void lcd_init()
{
    LCD_RS = 0;	          // write control bytes
	DelayMs(20);          // power on delay
    LCD_D4 = 1;	          // init!	
	LCD_D5 = 1; 
	LCD_STROBE;	
	DelayMs(10);
    LCD_STROBE;	          // init!	
	DelayUs(100);
    LCD_STROBE;	          // init!	
	DelayMs(10);
    LCD_D4 = 0;	         // set 4 bit mode
	LCD_STROBE;	
	DelayUs(60);
    lcd_write(0x0C);     // display on
	lcd_write(0x28);     // 4 bit mode, 1/16 duty, 5x8 font, 2lines
	lcd_write(0x04);    // entry mode advance cursor
    lcd_write(0x01);    // clear display and reset cursor
}


// *********CLEARS THE DISPLAY OF THE LCD ************

void lcd_clear()
{
	LCD_RS = 0;
	lcd_write(0x1);
	DelayMs(2);
}

// ***** ***********SENDS 1 BYTE OF DATA TO LCD *************** 

void wrdata(unsigned char c)
{
	LCD_RS = 1;	
	lcd_write(c);
}

// ***** SENDS COMMAND TO LCD ***** 

void wrcmd(unsigned char command)
{
LCD_RS=0;
lcd_write(command);
}

// ******   MOVES CURSOR OF THE LCD TO TARGET POSITION ***********

void lcdgoto(unsigned char pos, int line)
{
	LCD_RS = 0;
    switch (line){
    case 1:{lcd_write(0x80+pos);break;}
    case 2:{lcd_write(0xC0+pos);break;}
    case 3:{lcd_write(0X94+pos);break;}
    case 4:{lcd_write(0xD4+pos);break;}
    default:break;}

}

// **** WRITES DATA TO 4X20 LCD SCREEN. Line one starts at address 0x80, Line 2 at 0xC0....**** 
// ****************Specific to application

void wrstr(char *data,int pos,int line){
 switch(line){
 case 1:{wrcmd(0x80+pos);break;}
 case 2:{wrcmd(0xC0+pos);break;}
 case 3:{wrcmd(0X94+pos);break;}
 case 4:{wrcmd(0xD4+pos);break;}
 default:break;}
 while(*data!=0) 
 wrdata(*data++);
}

// delay functions

