/*************************************************** 
/* PROJECT:   DS18B20 1 WIRE BUS TEMPERATURE CONTROL 
/* BY:        SERGULEN YENIDOGAN 
/* DATE:      29.09.2014 
/* REVISION:  10.10.2014
/* VERSION:   V1.0
****************************************************/
#include    "htc.h"
#include	"stdio.h"
#include	"stdlib.h"
#include	"string.h"
#include    "stddef.h"


#define LCD_RS	RB0	// Register select
#define LCD_EN	RB1	// Enable
#define LCD_D4  RB4	// Data bits
#define LCD_D5  RB5	// Data bits
#define LCD_D6  RB6	// Data bits
#define LCD_D7  RB7	// Data bits
#define LCD_STROBE	((LCD_EN = 1),(LCD_EN=0))




#ifndef _XTAL_FREQ
#define _XTAL_FREQ	20000000	
#endif
#define	MHZ	*1000L		
#define	KHZ	*1			
#if	_XTAL_FREQ >= 12MHZ
#define	DelayUs(x)	{ unsigned char _dcnt; \
			  _dcnt = (x)*((_XTAL_FREQ)/(12MHZ)); \
			  while(--_dcnt != 0) \
				  continue; }
#else

#define	DelayUs(x)	{ unsigned char _dcnt; \
			  _dcnt = (x)/((12MHZ)/(_XTAL_FREQ))|1; \
			  while(--_dcnt != 0) \
				  continue; }
#endif


void DelayMs(int cnt)
{
	unsigned char	i;
	do {
		i = 4;
		do {
			DelayUs(245);
			
		} while(--i);
	} while(--cnt);
}

#define CMD_CONVERTTEMP     0x44          //THIS COMMAND IS USED TO START A TEMPERATURE CONVERSION, THAT IS STORED IN THE FIRST TWO BYTES OF THE
                                          //SCRATCHPAD. THE CONVERSION TIME IS RESOLUTION DEPENDENT, -LOOK AT TABLE SHOWN IN CHAPTER 3.6 . IF YOU
                                          //SEND READ SLOTS WHILE CONVERSION IS IN PROGRESS THE SLAVE RESPONDS WITH 0 (LOW) IF IT IS STILL IN PROGRESS OR 1
                                          //(HIGH) IF IT HAS FINISHED (ONLY AVAILABLE IF IT IS NOT PARASITE POWERED).

#define CMD_RSCRATCHPAD     0xBE          //THIS COMMAND ALLOWS THE MASTER TO READ THE CONTENTS OF THE SCRATCHPAD. IT IS 9-BYTES LONG AND IT IS
                                          //TRANSFERRED STARTING WITH THE LEAST SIGNIFICANT BYTE. IF ALL BYTES ARE NOT NEEDED TRANSFER CAN BE 
                                          //STOPPED BY MASTER ISSUING A RESET.
                                          
#define CMD_WSCRATCHPAD     0x4E          //THIS COMMAND ALLOWS MASTER TO WRITE 3 BYTES OF DATA TO THE SCRATCHPAD: TH, TL AND CONFIGURATION
                                          //REGISTERS. DATA MUST BE TRANSFERRED WITH LEAST SIGNIFICANT BIT FIRST.

#define CMD_CPYSCRATCHPAD   0x48          //THIS COMMAND COPIES THE CONTENTS OF TH, TL AND CONFIGURATION REGISTERS FROM THE SCRATCHPAD TO THE EEPROM.

#define CMD_RECEEPROM       0xB8          //THIS COMMAND RECALLS THE ALARM TRIGGER VALUES (TH AND TL) AND CONFIGURATION DATA FROM EEPROM AND
                                          //PLACES THEM IN THE SCRATCHPAD.

#define CMD_RPWRSUPPLY      0xB4          //THIS COMMAND IS USED TO DETERMINE IF A SLAVE IS EXTERNALLY POWERED OR USES PARASITE POWER. PARASITE
                                          //POWERED SLAVES WILL KEEP THE LINE LOW AFTER THE COMMAND, AND HIGH THE EXTERNALLY POWERED ONES.

#define CMD_SEARCHROM       0xF0          //USED TO IDENTIFY THE ROM CODES OF THE AVAILABLE SLAVES IN THE BUS, WHICH ALSO LETS THE MASTER DETERMINE
                                          //THE TOTAL NUMBER OF SLAVES.

#define CMD_READROM         0x33          //THIS COMMAND HAS THE SAME EFFECT AS SEARCH ROM, BUT IT CAN BE USED WHEN THERE IS ONLY ONE DEVICE IN THE
                                          //BUS. IF NOT, DATA COLLISION WILL OCCUR.

#define CMD_MATCHROM        0x55          //THIS COMMAND FOLLOWED BY A 64-BIT ROM CODE IS USED TO ADDRESS A SPECIFIC SLAVE IN THE BUS. IF A SLAVE
                                          //MATCHES THE CODE, IT WILL BE THE ONLY ONE TO RESPOND TO THE COMMANDS. THE OTHERS WILL WAIT FOR A RESET SEQUENCE.

#define CMD_SKIPROM         0xCC          //THIS IS USED TO ADDRESS ALL THE DEVICES IN THE BUS AT THE SAME TIME. THIS COULD BE USEFUL TO SEND COMMANDS
                                          //SUCH AS THE START OF TEMPERATURE CONVERSION (CONVERT T, 0X44) OR IF THERE IS ONLY ONE DEVICE IN THE BUS.

#define CMD_ALARMSEARCH     0xEC          //THIS COMMAND IS NEARLY THE SAME AS SEARCH ROM, EXCEPTING THAT ONLY DEVICES WITH A SET ALARM FLAG WILL RESPOND.

#define BUSPIN RD0                        //1 WIRE COMMUNICATION PIN

#define TRIS_BUSPIN TRISD0                //BUSPIN INPUT OR OUTPUT SELECTION REGISTER

#define THERM_DECIMAL_STEPS_12BIT 0.0625



unsigned char bus_reset(void)             //RESET BUS FOR NEW START
{
	unsigned char status;
    TRIS_BUSPIN=0;                       // SET BUSPIN TO OUTPUT
	BUSPIN=0;                            // OUTPUT LOW	                       
	DelayUs(480);                        // PULL LINE LOW AND WAIT FOR 480uS
	TRIS_BUSPIN=1;                       // RELEASE LINE AND WAIT FOR 60uS
	DelayUs(60); 
    status=BUSPIN;                           
	DelayUs(420);                       
	return status;                       // RETURN THE VALUE READ FROM THE PRESENCE PULSE (0=OK, 1=WRONG)
}


/***** 

IN THE WRITE OPERATION, WE ALWAYS HAVE TO PULL THE LINE LOW AT STARTUP. IF WE WANT TO WRITE 0 WE
WILL KEEP THE LINE LOW UNTIL THE END OF THE SEQUENCE (60S) BUT IF WE WANT TO WRITE 1, AFTER A
DELAY OF 1s WE WILL RELEASE IT.

*****/

void write_bit(unsigned char onebit)    
{
    	 
    
	if(onebit)
    {
    TRIS_BUSPIN=0;
    BUSPIN=0;
    DelayUs(1); 
    TRIS_BUSPIN=1;
    DelayUs(59); 
    } 
    else
    {
    TRIS_BUSPIN=0;
    BUSPIN=0;
    DelayUs(60);
    TRIS_BUSPIN=1;
    }
}
/****

ON THE OTHER SIDE, WE HAVE THE READ OPERATION. IT IS QUITE SIMILAR, BUT HAS SOME DIFFERENCES. WE
ALSO HAVE TO START PULLING THE LINE LOW FOR 1S. THEN WE HAVE TO RELEASE THE LINE AND WAIT FOR
14S MORE. AFTER THAT, WE CAN READ THE VALUE OF THE LINE, THAT WILL BE HIGH IF THERMOMETER 
TRANSMITS 1 AND LOW IF IT TRANSMITS 0. FINALLY, WE JUST HAVE TO
WAIT 45S MORE TO END THE 60S PERIOD. 
 
****/

unsigned char read_bit(void)
{
	unsigned char onebit=0;
    TRIS_BUSPIN=0;
	BUSPIN=0; 
	DelayUs(1); 
	TRIS_BUSPIN=1; 
	DelayUs(14);
	if(BUSPIN) 
    {
    onebit=1;
    }
	DelayUs(45);
	return onebit;
}
/****

NOW THAT WE CAN READ/WRITE INDIVIDUAL BITS, DOING THIS FOR BYTES IS QUITE EASY: MAKE LOOPS OF 8
CYCLES AND STORE THE RESULT IN A VARIABLE.
****/

unsigned char read_byte(void)
{
unsigned char i=8, onebyte=0;
while(i--)
{
onebyte>>=1;
onebyte|=read_bit()<<7;
}
return onebyte;
}

void write_byte(unsigned char onebyte)
{
unsigned char i=8;
while(i--){
write_bit(onebyte&1);               
onebyte>>=1;
}
}

unsigned char read_temperature(void){                 // Buffer length must be at least 12bytes long! ["+XXX.XXXX C"]
unsigned char LSB,MSB;
unsigned char digit=0;                                  //Reset, skip ROM and start temperature conversion
bus_reset();
write_byte(CMD_SKIPROM);
write_byte(CMD_CONVERTTEMP);
DelayMs(750);                                                      //Wait until conversion is complet        
bus_reset();                                          //Reset, skip ROM and send command to read Scratchpad
write_byte(CMD_SKIPROM);
write_byte(CMD_RSCRATCHPAD);
LSB=read_byte();                                      //Read Scratchpad (only 2 first bytes)
MSB=read_byte();
bus_reset();                                          //Store temperature integer digits and decimal digits
digit=LSB>>4;
digit|=(MSB&15)<<4;
return digit;
}


void lcd_write(unsigned char c)
{   
	if(c & 0x80) LCD_D7=1; else LCD_D7=0;
	if(c & 0x40) LCD_D6=1; else LCD_D6=0;
	if(c & 0x20) LCD_D5=1; else LCD_D5=0;
	if(c & 0x10) LCD_D4=1; else LCD_D4=0;
	LCD_STROBE;
	if(c & 0x08) LCD_D7=1; else LCD_D7=0;
	if(c & 0x04) LCD_D6=1; else LCD_D6=0;
	if(c & 0x02) LCD_D5=1; else LCD_D5=0;
	if(c & 0x01) LCD_D4=1; else LCD_D4=0;
	LCD_STROBE;	
	DelayUs(60);
}


void lcd_init()
{
    LCD_RS = 0;	          // write control bytes
	DelayMs(20);          // power on delay
    LCD_D4 = 1;	          // init!	
	LCD_D5 = 1; 
	LCD_STROBE;	
	DelayMs(10);
    LCD_STROBE;	          // init!	
	DelayUs(100);
    LCD_STROBE;	          // init!	
	DelayMs(10);
    LCD_D4 = 0;	         // set 4 bit mode
	LCD_STROBE;	
	DelayUs(60);
    lcd_write(0x0C);     // display on
	lcd_write(0x28);     // 4 bit mode, 1/16 duty, 5x8 font, 2lines
	lcd_write(0x04);    // entry mode advance cursor
    lcd_write(0x01);    // clear display and reset cursor
}


// *********CLEARS THE DISPLAY OF THE LCD ************

void lcd_clear()
{
	LCD_RS = 0;
	lcd_write(0x1);
	DelayMs(2);
}

// ***** ***********SENDS 1 BYTE OF DATA TO LCD *************** 

void wrdata(unsigned char c)
{
	LCD_RS = 1;	
	lcd_write(c);
}

// ***** SENDS COMMAND TO LCD ***** 

void wrcmd(unsigned char command)
{
LCD_RS=0;
lcd_write(command);
}

// ******   MOVES CURSOR OF THE LCD TO TARGET POSITION ***********

void lcdgoto(unsigned char pos, int line)
{
	LCD_RS = 0;
    switch (line){
    case 1:{lcd_write(0x80+pos);break;}
    case 2:{lcd_write(0xC0+pos);break;}
    case 3:{lcd_write(0X94+pos);break;}
    case 4:{lcd_write(0xD4+pos);break;}
    default:break;}

}

// **** WRITES DATA TO 4X20 LCD SCREEN. Line one starts at address 0x80, Line 2 at 0xC0....**** 
// ****************Specific to application

void wrstr(char *data,int pos,int line){
 switch(line){
 case 1:{wrcmd(0x80+pos);break;}
 case 2:{wrcmd(0xC0+pos);break;}
 case 3:{wrcmd(0X94+pos);break;}
 case 4:{wrcmd(0xD4+pos);break;}
 default:break;}
 while(*data!=0) 
 wrdata(*data++);
}
void main (void){
unsigned char lcddata[20];
    TMR0=133;			
	TMR0IF=0;
	TMR0IE=0;			
//	T0CON=0xC2;			
	T0CS=0;
	PSA=0;
	PS2=1;	//prescale is 1:32
	PS1=0;
	PS0=0;
	T1CON=0x30;
	T1CKPS0=1;
	T1CKPS1=1;
	GIE = 0;
    ADCON1  = 0b00001000 ;
    TRISA   = 0b11111111;
    TRISC   = PORTC=0b00000000;
    TRISB   = PORTB=0b00000000;
    TRISD   = PORTD=0b00000000;
    TRISE   = PORTE=0b00000000;
 
lcd_init();
signed char dert=0;

while (1)
{
dert=read_temperature();
sprintf(lcddata,"%d" ,dert);
wrstr(lcddata,0,1);
}
}
/*                 */ 